/* File:      chain.C
   Date:      6/22/02 (written fall 2001)
   Author:    C. Paciorek
   Purpose:   basic code for running MCMC - initialization and iteration
   Compile:   g++ -c 
   Usage:     
   Arguments: 
   Requires:  chain.h,base.h,parm.h,dataset.h,transform.h,generic_functions.h,mat_pp.h (in base.h)
   Reference: 
*/

#include "base.h"
#include "chain.h"
#include "cov_mat.h"       // for accessing kernels
#include "parm.h"
#include "dataset.h"
#include "transform.h"
#include "density.h"
#include "locations.h"        // for permuting initial psi values
#include "permtools.h"
#include "ran1.h"

void dumb(void){
  int i=6;
  i++;
}

chain::chain(int num_gridpoints_in,string dat_fn_in,string years_fn_in,string init_fn_in,string final_fn_in,string parm_fn_in,string out_fn_in,int num_its_in,int storage_interval_in,int storage_space_in,int fastparm_interval_in,string datacorr_fn_in){

  // some of the variable names in here are misnomers: kappa hyp is not logged anymore and kappa, eigenvec,delta,eigenval all have uniform priors, though delta and eigenval are on log scale
  int j;

  int alpha_init_mode,beta_init_mode,eta_init_mode, eigenval1_init_mode,eigenval2_init_mode, eigenvec1_init_mode; // eigenvec2_init_mode;
  double alpha_logkappa_init,alpha_mu_init,alpha_logsigma_init,alpha_logkappa_mean,alpha_logkappa_sd,alpha_mu_mean,alpha_mu_sd,alpha_logsigma_mean,alpha_logsigma_sd,alpha_theta_propsd,alpha_theta_propsd_subgroup,alpha_mu_propsd,alpha_logsigma_propsd;
  int alpha_subgroup_size;
  double alpha_logkappa_propsd;
  double beta_logkappa_init,beta_mu_init,beta_logsigma_init,beta_logkappa_mean,beta_logkappa_sd,beta_mu_mean,beta_mu_sd,beta_logsigma_mean,beta_logsigma_sd,beta_theta_propsd,beta_theta_propsd_subgroup,beta_mu_propsd,beta_logsigma_propsd;
  int beta_subgroup_size;
  double beta_logkappa_propsd;
  double eta_logkappa_init,eta_mu_init,eta_logsigma_init,eta_logkappa_mean,eta_logkappa_sd,eta_mu_mean,eta_mu_sd,eta_logsigma_mean,eta_logsigma_sd,eta_theta_propsd,eta_theta_propsd_subgroup,eta_mu_propsd,eta_logsigma_propsd;  // location variances
  int eta_subgroup_size;
  double eta_logkappa_propsd;

  // used for proposing kappa on its own 
  double alpha_logkappa_propsd2, beta_logkappa_propsd2, eta_logkappa_propsd2;

  double alpha_ratio, beta_ratio, eta_ratio; // ratio of sds of log(sigma), log(kappa) for generalized S/D proposals

  double delta_init, delta_mean, delta_sd, delta_propsd; // noise variance

  double nu_init, nu_low, nu_high, nu_propsd;  // Matern smoothness for data correlation

  // for stationary data covariance (& nonstat with one kappa)
  double data_kappa_init, data_kappa_mean, data_kappa_sd;
  double data_kappa_propsd;

  // for nonstationary data covariance

  // basis kernel variables

  double eigenval1_mean, eigenval1_sd,eigenval1_propsd;
  double eigenval2_mean, eigenval2_sd,eigenval2_propsd;
  double eigenvec1_mean, eigenvec1_sd;
  double eigenvec1_propsd;


  // full kernel variables
  /*
  double eigenval1_mu_init;
  double eigenval1_logsigma_init;
  double eigenval1_logkappa_init;
  double eigenval2_mu_init;
  double eigenval2_logsigma_init;
  double eigenval2_logkappa_init;

  double eigenvec1_mu_init;
  double eigenvec1_logsigma_init;
  double eigenvec1_logkappa_init;
  double eigenvec2_mu_init;
  double eigenvec2_logsigma_init;
  double eigenvec2_logkappa_init;

   double eigenval1_mu_mean;
  double eigenval1_mu_sd;
  double eigenval1_mu_propsd;
   double eigenval2_mu_mean;
  double eigenval2_mu_sd;
  double eigenval2_mu_propsd;

  double eigenvec1_mu_mean;
  double eigenvec1_mu_sd;
  double eigenvec1_mu_propsd;
  double eigenvec2_mu_mean;
  double eigenvec2_mu_sd;
  double eigenvec2_mu_propsd;

  double eigenval1_logsigma_mean;
  double eigenval1_logsigma_sd;
  double eigenval1_logsigma_propsd;
  double eigenval2_logsigma_mean;
  double eigenval2_logsigma_sd;
  double eigenval2_logsigma_propsd;

  double eigenvec1_logsigma_mean;
  double eigenvec1_logsigma_sd;
  double eigenvec1_logsigma_propsd;
  double eigenvec2_logsigma_mean;
  double eigenvec2_logsigma_sd;
  double eigenvec2_logsigma_propsd;

  double eigenval1_logkappa_mean;
  double eigenval1_logkappa_sd;
  double eigenval2_logkappa_mean;
  double eigenval2_logkappa_sd;

  double eigenvec1_logkappa_mean;
  double eigenvec1_logkappa_sd;
  double eigenvec2_logkappa_mean;
  double eigenvec2_logkappa_sd;

  double eigenval1_theta_propsd;
  double eigenvec1_theta_propsd;
  double eigenval2_theta_propsd;
  double eigenvec2_theta_propsd;
  double eigenval1_theta_propsd_subgroup;
  double eigenvec1_theta_propsd_subgroup;
  double eigenval2_theta_propsd_subgroup;
  double eigenvec2_theta_propsd_subgroup;

  int eigenval1_subgroup_size;
  int eigenval2_subgroup_size;
  int eigenvec1_subgroup_size;
  int eigenvec2_subgroup_size;

  double eigenval1_logkappa_propsd;
  double eigenvec1_logkappa_propsd;
  double eigenval2_logkappa_propsd;
  double eigenvec2_logkappa_propsd;
  */
  /* eigenvec2 stuff not used with basis kernel approach */

  num_gridpoints=num_gridpoints_in;
  final_fn=final_fn_in;

  out_fn=out_fn_in;
  num_its=num_its_in;

  storage_interval=storage_interval_in;
  storage_space=storage_space_in;
  fastparm_interval=fastparm_interval_in;

  it_run=0;

  block=1;

  // used in full kernel approach to generalize for variable dimensions
  // int num_eigenvals=2;
  // int num_eigenvecs=2;


  //////////////////////////////////////////////////
  // create storage space for parameter iterates
  //////////////////////////////////////////////////

  storage1=new mat_pp<double>(storage_space,17);
  storage2=new mat_pp<double>(storage_space,num_gridpoints);
  storage3=new mat_pp<double>(storage_space,num_gridpoints);
  storage4=new mat_pp<double>(storage_space,num_gridpoints);
  if(prior==NON_STATIONARY){
    storage5=new mat_pp<double>(storage_space,2+locations->num_kernels*3);

    // storage6 is temporary storage for kernel iterates
    /*
    storage6=new mat_pp<double>(storage_space,15*4);
    */

    // for full kernel approach
    //    storage5=new mat_pp<double>(storage_space,num_eigenvals*num_gridpoints);
    //storage6=new mat_pp<double>(storage_space,num_eigenvecs*num_gridpoints);
    //storage7=new mat_pp<double>(storage_space,12);
  }
  
  // read in which locations to store kernels for in storage6
  /*
  mat_pp<int> tmplocs(1,15);
  tmplocs.read("/TOIL-U2/paciorek/research/dat/kernstore.dat",BY_ROW,RAGGED);    // reads in the latitudes
  tmplocs.getvec(0,ROW,&kernlocs);           
  */

  ////////////////////////////////
  // Parameter initialization
  ////////////////////////////////

  ifstream init(init_fn_in.c_str());
  alpha_init=new double[num_gridpoints];
  init >> alpha_init_mode;   // 0 = psi init; 1 = theta init - all same init vals; 2=theta init, all different; will be 2 if I'm using output from previous runs
  if(alpha_init_mode==THETA_REP){
    init >> alpha_init[0];
    for(j=1;j<num_gridpoints;j++){ 
      alpha_init[j]=alpha_init[0];  
    }
  }
  else{
    for(j=0;j<num_gridpoints;j++){ 
      init >> alpha_init[j];  
    }
    if(alpha_init_mode==KSI_NOREP){
      permrv(&num_gridpoints,alpha_init,locations->perm,PERMSTART1);
    }
  }
  init >> alpha_mu_init; 
  init >> alpha_logsigma_init;
  init >> alpha_logkappa_init;

  cout << "Alpha initial values: " << alpha_mu_init << " " << alpha_logsigma_init << " " << alpha_logkappa_init << endl;
  for(j=0;j<num_gridpoints;j++){ 
    cout << alpha_init[j] << " ";  
  }
  cout << endl;

  beta_init=new double[num_gridpoints];
  init >> beta_init_mode;   // 0 = psi init; 1 = theta init - all same init vals; 2=theta init, all different; will be 2 if I'm using output from previous runs
  if(beta_init_mode==THETA_REP){
    init >> beta_init[0];
    for(j=1;j<num_gridpoints;j++){ 
      beta_init[j]=beta_init[0];  
    }
  }
  else{
    for(j=0;j<num_gridpoints;j++){ 
      init >> beta_init[j];  
    }
    if(beta_init_mode==KSI_NOREP){
      permrv(&num_gridpoints,beta_init,locations->perm,PERMSTART1);
    }
  }
  init >> beta_mu_init; 
  init >> beta_logsigma_init;
  init >> beta_logkappa_init;

  cout << "Beta initial values: " << beta_mu_init << " " << beta_logsigma_init << " " << beta_logkappa_init << endl;
  for(j=0;j<num_gridpoints;j++){ 
    cout << beta_init[j] << " ";  
  }
  cout << endl;

  eta_init=new double[num_gridpoints];
  init >> eta_init_mode;   // 0 = psi init; 1 = theta init - all same init vals; 2=theta init, all different; will be 2 if I'm using output from previous runs
  if(eta_init_mode==THETA_REP){
    init >> eta_init[0];
    for(j=1;j<num_gridpoints;j++){ 
      eta_init[j]=eta_init[0];  
    }
  }
  else{
    for(j=0;j<num_gridpoints;j++){ 
      init >> eta_init[j];  
    }
    if(eta_init_mode==KSI_NOREP){
      permrv(&num_gridpoints,eta_init,locations->perm,PERMSTART1);
    }
  }
  init >> eta_mu_init; 
  init >> eta_logsigma_init;
  init >> eta_logkappa_init;

  cout << "Eta initial values: " << eta_mu_init << " " << eta_logsigma_init << " " << eta_logkappa_init << endl;
  for(j=0;j<num_gridpoints;j++){ 
    cout << eta_init[j] << " ";  
  }
  cout << endl;

  init >> delta_init;
  cout << "Delta init " << delta_init << endl;

  init >> nu_init;
  cout << "Nu init " << nu_init << endl;

  if(prior==STATIONARY){
    init >> data_kappa_init;
    cout << "Data kappa init " << data_kappa_init << endl;
  }

  // new code for basis kernel approach

  if(prior==NON_STATIONARY){

    init >> data_kappa_init;
    cout << "Data kappa init " << data_kappa_init << endl;

    // eigenval initialization
    eigenval1_init=new double[locations->num_kernels];
    init >> eigenval1_init_mode;   // 0 = psi init; 1 = theta init - all same init vals; 2=theta init, all different; will be 2 if I'm using output from previous runs
    if(eigenval1_init_mode==THETA_REP){
      init >> eigenval1_init[0];
      for(j=1;j<locations->num_kernels;j++){ 
	eigenval1_init[j]=eigenval1_init[0];  
      }
    }
    else{
      for(j=0;j<locations->num_kernels;j++){ 
	init >> eigenval1_init[j];  
      }
    }

    cout << "Eigenval1 init ";
    for(j=0;j<locations->num_kernels;j++){ 
      cout << eigenval1_init[j] << " ";  
    }
    cout << endl;

 
    eigenval2_init=new double[locations->num_kernels];
    init >> eigenval2_init_mode;   // 0 = psi init; 1 = theta init - all same init vals; 2=theta init, all different; will be 2 if I'm using output from previous runs
    if(eigenval2_init_mode==THETA_REP){
      init >> eigenval2_init[0];
      for(j=1;j<locations->num_kernels;j++){ 
	eigenval2_init[j]=eigenval2_init[0];  
      }
    }
    else{
      for(j=0;j<locations->num_kernels;j++){ 
	init >> eigenval2_init[j];  
      }
    }
 
    cout << "Eigenval2 init ";
    for(j=0;j<locations->num_kernels;j++){ 
      cout << eigenval2_init[j] << " ";  
    }
    cout << endl;

    eigenvec1_init=new double[locations->num_kernels];
    init >> eigenvec1_init_mode;   // 0 = psi init; 1 = theta init - all same init vals; 2=theta init, all different; will be 2 if I'm using output from previous runs
    if(eigenvec1_init_mode==THETA_REP){
      init >> eigenvec1_init[0];
      for(j=1;j<locations->num_kernels;j++){ 
	eigenvec1_init[j]=eigenvec1_init[0];  
      }
    }
    else{
      for(j=0;j<locations->num_kernels;j++){ 
	init >> eigenvec1_init[j];  
      }
    }
    cout << "Eigenvec1 init ";
    for(j=0;j<locations->num_kernels;j++){ 
      cout << eigenvec1_init[j] << " ";  
    }
    cout << endl;
  }


  // old code for full model for kernels
  /*
  if(prior==NON_STATIONARY){
    // eigenval initialization
    eigenval1_init=new double[num_gridpoints];
    init >> eigenval1_init_mode;   // 0 = psi init; 1 = theta init - all same init vals; 2=theta init, all different; will be 2 if I'm using output from previous runs
    if(eigenval1_init_mode==THETA_REP){
      init >> eigenval1_init[0];
      for(j=1;j<num_gridpoints;j++){ 
	eigenval1_init[j]=eigenval1_init[0];  
      }
    }
    else{
      for(j=0;j<num_gridpoints;j++){ 
	init >> eigenval1_init[j];  
      }
      if(eigenval1_init_mode==KSI_NOREP){
	permrv(&num_gridpoints,eigenval1_init,locations->perm,PERMSTART1);
      }
    }
    init >> eigenval1_mu_init; 
    init >> eigenval1_logsigma_init;
    init >> eigenval1_logkappa_init;
 
    eigenval2_init=new double[num_gridpoints];
    init >> eigenval2_init_mode;   // 0 = psi init; 1 = theta init - all same init vals; 2=theta init, all different; will be 2 if I'm using output from previous runs
    if(eigenval2_init_mode==THETA_REP){
      init >> eigenval2_init[0];
      for(j=1;j<num_gridpoints;j++){ 
	eigenval2_init[j]=eigenval2_init[0];  
      }
    }
    else{
      for(j=0;j<num_gridpoints;j++){ 
	init >> eigenval2_init[j];  
      }
      if(eigenval2_init_mode==KSI_NOREP){
	permrv(&num_gridpoints,eigenval2_init,locations->perm,PERMSTART1);
      }
    }
    init >> eigenval2_mu_init; 
    init >> eigenval2_logsigma_init;
    init >> eigenval2_logkappa_init;
 
    eigenvec1_init=new double[num_gridpoints];
    init >> eigenvec1_init_mode;   // 0 = psi init; 1 = theta init - all same init vals; 2=theta init, all different; will be 2 if I'm using output from previous runs
    if(eigenvec1_init_mode==THETA_REP){
      init >> eigenvec1_init[0];
      for(j=1;j<num_gridpoints;j++){ 
	eigenvec1_init[j]=eigenvec1_init[0];  
      }
      if(eigenvec1_init_mode==KSI_NOREP){
	permrv(&num_gridpoints,eigenvec1_init,locations->perm,PERMSTART1);
      }
    }
    else{
      for(j=0;j<num_gridpoints;j++){ 
	init >> eigenvec1_init[j];  
      }
    }
    init >> eigenvec1_mu_init; 
    init >> eigenvec1_logsigma_init;
    init >> eigenvec1_logkappa_init;


    eigenvec2_init=new double[num_gridpoints];
    init >> eigenvec2_init_mode;   // 0 = psi init; 1 = theta init - all same init vals; 2=theta init, all different; will be 2 if I'm using output from previous runs
    if(eigenvec2_init_mode==THETA_REP){
      init >> eigenvec2_init[0];
      for(j=1;j<num_gridpoints;j++){ 
	eigenvec2_init[j]=eigenvec2_init[0];  
      }
    }
    else{
      for(j=0;j<num_gridpoints;j++){ 
	init >> eigenvec2_init[j];  
      }
      if(eigenvec2_init_mode==KSI_NOREP){
	permrv(&num_gridpoints,eigenvec2_init,locations->perm,PERMSTART1);
      }
    }
    init >> eigenvec2_mu_init; 
    init >> eigenvec2_logsigma_init;
    init >> eigenvec2_logkappa_init;
  }
  */

  ///////////////////////////////////
  /// random seed stuff
  ///////////////////////////////////

  // initialize random variable
  idum= new int;
  char line[1024];
  init >> *idum;
  init.getline(line,sizeof(line));
  ran1(idum,READ,line);
  
  // fill ran1 stored values if not yet initialized
  if(*idum<=0){
    ran1(idum,GENERATE,"a");
  }

  /////////////////////////////////////
  // Fixed hyperparameter values
  /////////////////////////////////////
  int on;

  ifstream parm(parm_fn_in.c_str());
  parm >> alpha_mu_mean;
  parm >> alpha_mu_sd;
  parm >> alpha_logsigma_mean;
  parm >> alpha_logsigma_sd;
  parm >> alpha_logkappa_mean;
  parm >> alpha_logkappa_sd;

  parm >> alpha_theta_propsd;
  parm >> on;
  alpha_theta_propsd*=(double) on;
  parm >> alpha_theta_propsd_subgroup;
  parm >> alpha_subgroup_size;
  alpha_theta_propsd_subgroup*=(double)(alpha_subgroup_size>0);
  parm >> alpha_mu_propsd;
  parm >> on;
  alpha_mu_propsd*=(double)on;
  parm >> alpha_logsigma_propsd;
  parm >> on;
  alpha_logsigma_propsd*=(double)on;
  parm >> alpha_logkappa_propsd;
  parm >> on;
  alpha_logkappa_propsd*=(double)on;
  parm >> alpha_logkappa_propsd2;
  parm >> on;
  alpha_logkappa_propsd2*=(double)on;
  parm >> alpha_ratio;

  parm >> beta_mu_mean;
  parm >> beta_mu_sd;
  parm >> beta_logsigma_mean;
  parm >> beta_logsigma_sd;
  parm >> beta_logkappa_mean;
  parm >> beta_logkappa_sd;

  parm >> beta_theta_propsd;
  parm >> on;
  beta_theta_propsd*=(double) on;
  parm >> beta_theta_propsd_subgroup;
  parm >> beta_subgroup_size;
  beta_theta_propsd_subgroup*=(double)(beta_subgroup_size>0);
  parm >> beta_mu_propsd;
  parm >> on;
  beta_mu_propsd*=(double)on;
  parm >> beta_logsigma_propsd;
  parm >> on;
  beta_logsigma_propsd*=(double)on;
  parm >> beta_logkappa_propsd;
  parm >> on;
  beta_logkappa_propsd*=(double)on;
  parm >> beta_logkappa_propsd2;
  parm >> on;
  beta_logkappa_propsd2*=(double)on;
  parm >> beta_ratio;

  parm >> eta_mu_mean;
  parm >> eta_mu_sd;
  parm >> eta_logsigma_mean;
  parm >> eta_logsigma_sd;
  parm >> eta_logkappa_mean;
  parm >> eta_logkappa_sd;

  parm >> eta_theta_propsd;
  parm >> on;
  eta_theta_propsd*=(double) on;
  parm >> eta_theta_propsd_subgroup;
  parm >> eta_subgroup_size;
  eta_theta_propsd_subgroup*=(double)(eta_subgroup_size>0);
  parm >> eta_mu_propsd;
  parm >> on;
  eta_mu_propsd*=(double)on;
  parm >> eta_logsigma_propsd;
  parm >> on;
  eta_logsigma_propsd*=(double)on;
  parm >> eta_logkappa_propsd;
  parm >> on;
  eta_logkappa_propsd*=(double)on;
  parm >> eta_logkappa_propsd2;
  parm >> on;
  eta_logkappa_propsd2*=(double)on;
  parm >> eta_ratio;

  parm >> delta_mean;
  parm >> delta_sd;
  parm >> delta_propsd;
  parm >> on;
  delta_propsd*=(double)on;

  parm >> nu_low;
  parm >> nu_high;
  parm >> nu_propsd;
  parm >> on;
  nu_propsd*=(double)on;

  if(prior==STATIONARY){
    parm >> data_kappa_mean;
    parm >> data_kappa_sd;
    parm >> data_kappa_propsd;
    parm >> on;
    data_kappa_propsd*=(double)on;
  }
  if(prior==NON_STATIONARY){

    parm >> data_kappa_mean;
    parm >> data_kappa_sd;
    parm >> data_kappa_propsd;
    parm >> on;
    data_kappa_propsd*=(double)on;

    parm >> eigenval1_mean;
    parm >> eigenval1_sd;
    parm >> eigenval1_propsd;
    parm >> on;
    eigenval1_propsd*=(double)on;
   
    parm >> eigenval2_mean;
    parm >> eigenval2_sd;
    parm >> eigenval2_propsd;
    parm >> on;
    eigenval2_propsd*=(double)on;

    parm >> eigenvec1_mean;
    parm >> eigenvec1_sd;
    parm >> eigenvec1_propsd;
    parm >> on;
    eigenvec1_propsd*=(double)on;

    /* code for full kernel approach
    parm >> eigenval1_mu_mean;
    parm >> eigenval1_mu_sd;
    parm >> eigenval1_logsigma_mean;
    parm >> eigenval1_logsigma_sd;
    parm >> eigenval1_logkappa_mean;
    parm >> eigenval1_logkappa_sd;
    
    parm >> eigenval1_theta_propsd;
    parm >> eigenval1_theta_propsd_subgroup;
    parm >> eigenval1_subgroup_size;
    eigenval1_theta_propsd_subgroup*=(double)(eigenval1_subgroup_size>0);
    parm >> eigenval1_mu_propsd;
    parm >> on;
    eigenval1_mu_propsd*=(double)on;
    parm >> eigenval1_logsigma_propsd;
    parm >> on;
    eigenval1_logsigma_propsd*=(double)on;
    parm >> eigenval1_logkappa_propsd;
    parm >> on;
    eigenval1_logkappa_propsd*=(double)on;
    
    parm >> eigenval2_mu_mean;
    parm >> eigenval2_mu_sd;
    parm >> eigenval2_logsigma_mean;
    parm >> eigenval2_logsigma_sd;
    parm >> eigenval2_logkappa_mean;
    parm >> eigenval2_logkappa_sd;
    
    parm >> eigenval2_theta_propsd;
    parm >> eigenval2_theta_propsd_subgroup;
    parm >> eigenval2_subgroup_size;
    eigenval2_theta_propsd_subgroup*=(double)(eigenval2_subgroup_size>0);
    parm >> eigenval2_mu_propsd;
    parm >> on;
    eigenval2_mu_propsd*=(double)on;
    parm >> eigenval2_logsigma_propsd;
    parm >> on;
    eigenval2_logsigma_propsd*=(double)on;
    parm >> eigenval2_logkappa_propsd;
    parm >> on;
    eigenval2_logkappa_propsd*=(double)on;
    
    parm >> eigenvec1_mu_mean;
    parm >> eigenvec1_mu_sd;
    parm >> eigenvec1_logsigma_mean;
    parm >> eigenvec1_logsigma_sd;
    parm >> eigenvec1_logkappa_mean;
    parm >> eigenvec1_logkappa_sd;
    
    parm >> eigenvec1_theta_propsd;
    parm >> eigenvec1_theta_propsd_subgroup;
    parm >> eigenvec1_subgroup_size;
    eigenvec1_theta_propsd_subgroup*=(double)(eigenvec1_subgroup_size>0);
    parm >> eigenvec1_mu_propsd;
    parm >> on;
    eigenvec1_mu_propsd*=(double)on;
    parm >> eigenvec1_logsigma_propsd;
    parm >> on;
    eigenvec1_logsigma_propsd*=(double)on;
    parm >> eigenvec1_logkappa_propsd;
    parm >> on;
    eigenvec1_logkappa_propsd*=(double)on;
    
    parm >> eigenvec2_mu_mean;
    parm >> eigenvec2_mu_sd;
    parm >> eigenvec2_logsigma_mean;
    parm >> eigenvec2_logsigma_sd;
    parm >> eigenvec2_logkappa_mean;
    parm >> eigenvec2_logkappa_sd;
    
    parm >> eigenvec2_theta_propsd;
    parm >> eigenvec2_theta_propsd_subgroup;
    parm >> eigenvec2_subgroup_size;
    eigenvec2_theta_propsd_subgroup*=(double)(eigenvec2_subgroup_size>0);
    parm >> eigenvec2_mu_propsd;
    parm >> on;
    eigenvec2_mu_propsd*=(double)on;
    parm >> eigenvec2_logsigma_propsd;
    parm >> on;
    eigenvec2_logsigma_propsd*=(double)on;
    parm >> eigenvec2_logkappa_propsd;
    parm >> on;
    eigenvec2_logkappa_propsd*=(double)on;
  
    */
  }

  /* ALPHA */
  alpha = new spatial_parm(ALPHA,                    // which
			   alpha_init_mode,
			   alpha_init,       
			   alpha_mu_init,
			   alpha_logsigma_init,
			   alpha_logkappa_init,
			   alpha_mu_mean,
			   alpha_mu_sd,
			   alpha_logsigma_mean,
			   alpha_logsigma_sd,
			   alpha_logkappa_mean,
			   alpha_logkappa_sd,
			   alpha_theta_propsd,
			   alpha_theta_propsd_subgroup,
			   alpha_subgroup_size,
			   alpha_mu_propsd,
			   alpha_logsigma_propsd,
			   alpha_logkappa_propsd,
                           alpha_ratio,
			   //  			   &log_transform);
    			   &identity,
                           alpha_logkappa_propsd2
    );

  /* BETA */
  beta = new spatial_parm(BETA,                      // which
			  beta_init_mode,
			  beta_init,               
			  beta_mu_init,
			  beta_logsigma_init,
			  beta_logkappa_init,
			  beta_mu_mean,
			  beta_mu_sd,
			  beta_logsigma_mean,
			  beta_logsigma_sd,
			  beta_logkappa_mean,
			  beta_logkappa_sd,
			  beta_theta_propsd,
			  beta_theta_propsd_subgroup,
			  beta_subgroup_size,
			  beta_mu_propsd,
			  beta_logsigma_propsd,
			  beta_logkappa_propsd,
                          beta_ratio,
			  &identity  ,                         
                          beta_logkappa_propsd2
    );
  /* ETA */
  eta = new spatial_parm(ETA,               // which
			   eta_init_mode,
			   eta_init,               
			   eta_mu_init,
			   eta_logsigma_init,
			   eta_logkappa_init,
			   eta_mu_mean,
			   eta_mu_sd,
			   eta_logsigma_mean,
			   eta_logsigma_sd,
			   eta_logkappa_mean,
			   eta_logkappa_sd,
			   eta_theta_propsd,
			   eta_theta_propsd_subgroup,
			 eta_subgroup_size,
			   eta_mu_propsd,
			   eta_logsigma_propsd,
			   eta_logkappa_propsd,
                         eta_ratio,
			 &log_transform,
                         eta_logkappa_propsd2
    );
  eta->add_child(alpha,beta);
  if(prior==STATIONARY){
    data_kappa = new nonspatial_parm(DATA_KAPPA,data_kappa_init,data_kappa_mean,data_kappa_sd,data_kappa_propsd,&uniform_dens,&log_transform);  
  }
  if(prior==NON_STATIONARY){

    // basis kernel approach
    data_kappa = new nonspatial_parm(DATA_EIG_KAPPA,data_kappa_init,data_kappa_mean,data_kappa_sd,data_kappa_propsd,&uniform_dens,&log_transform); 
    eigenval1vec=new nonspatial_vecparm(DATA_COVAR_EIGVAL,               // which
                                     locations->num_kernels,
                                     eigenval1_init,   
                                     eigenval1_mean,
                                     eigenval1_sd,
                                     eigenval1_propsd,
                                        &uniform_dens,
                                     &log_transform);
    eigenval2vec=new nonspatial_vecparm(DATA_COVAR_EIGVAL,               // which
                                     locations->num_kernels,
                                     eigenval2_init,   
                                     eigenval2_mean,
                                     eigenval2_sd,
                                     eigenval2_propsd,
                                        &uniform_dens,
                                     &log_transform);
    eigenvec1vec=new nonspatial_vecparm(DATA_COVAR_EIGVEC,               // which
                                     locations->num_kernels,
                                     eigenvec1_init,   
                                        eigenvec1_mean,
                                        eigenvec1_sd,
                                     eigenvec1_propsd,
                                        &uniform_dens,
                                     &identity);

    eigenval1vec->add_child(alpha,beta);
    eigenval2vec->add_child(alpha,beta);
    eigenvec1vec->add_child(alpha,beta);
    //data_kappavec->add_child(alpha,beta);

    /* code for full kernel approach
    eigenval1=new spatial_parm(DATA_COVAR_EIGVAL,               // which
			       eigenval1_init_mode,
			       eigenval1_init,               
			       eigenval1_mu_init,
			       eigenval1_logsigma_init,
			       eigenval1_logkappa_init,
			       eigenval1_mu_mean,
			       eigenval1_mu_sd,
			       eigenval1_logsigma_mean,
			       eigenval1_logsigma_sd,
			       eigenval1_logkappa_mean,
			       eigenval1_logkappa_sd,
			       eigenval1_theta_propsd,
			       eigenval1_theta_propsd_subgroup,
			       eigenval1_subgroup_size,
			       eigenval1_mu_propsd,
			       eigenval1_logsigma_propsd,
			       eigenval1_logkappa_propsd,
			       &log_transform);
    eigenval2=new spatial_parm(DATA_COVAR_EIGVAL,               // which
			       eigenval2_init_mode,
			       eigenval2_init,               
			       eigenval2_mu_init,
			       eigenval2_logsigma_init,
			       eigenval2_logkappa_init,
			       eigenval2_mu_mean,
			       eigenval2_mu_sd,
			       eigenval2_logsigma_mean,
			       eigenval2_logsigma_sd,
			       eigenval2_logkappa_mean,
			       eigenval2_logkappa_sd,
			       eigenval2_theta_propsd,
			       eigenval2_theta_propsd_subgroup,
			       eigenval2_subgroup_size,
			       eigenval2_mu_propsd,
			       eigenval2_logsigma_propsd,
			       eigenval2_logkappa_propsd,
			       &log_transform);
    eigenvec1=new spatial_parm(DATA_COVAR_EIGVEC,               // which
			       eigenvec1_init_mode,
			       eigenvec1_init,               
			       eigenvec1_mu_init,
			       eigenvec1_logsigma_init,
			       eigenvec1_logkappa_init,
			       eigenvec1_mu_mean,
			       eigenvec1_mu_sd,
			       eigenvec1_logsigma_mean,
			       eigenvec1_logsigma_sd,
			       eigenvec1_logkappa_mean,
			       eigenvec1_logkappa_sd,
			       eigenvec1_theta_propsd,
			       eigenvec1_theta_propsd_subgroup,
			       eigenvec1_subgroup_size,
			       eigenvec1_mu_propsd,
			       eigenvec1_logsigma_propsd,
			       eigenvec1_logkappa_propsd,
			       &identity);
    eigenvec2=new spatial_parm(DATA_COVAR_EIGVEC,               // which
			       eigenvec2_init_mode,
			       eigenvec2_init,               
			       eigenvec2_mu_init,
			       eigenvec2_logsigma_init,
			       eigenvec2_logkappa_init,
			       eigenvec2_mu_mean,
			       eigenvec2_mu_sd,
			       eigenvec2_logsigma_mean,
			       eigenvec2_logsigma_sd,
			       eigenvec2_logkappa_mean,
			       eigenvec2_logkappa_sd,
			       eigenvec2_theta_propsd,
			       eigenvec2_theta_propsd_subgroup,
			       eigenvec2_subgroup_size,
			       eigenvec2_mu_propsd,
			       eigenvec2_logsigma_propsd,
			       eigenvec2_logkappa_propsd,
			       &identity);
    eigenval1->add_child(alpha,beta);
    eigenval2->add_child(alpha,beta);
    eigenvec1->add_child(alpha,beta);
    eigenvec2->add_child(alpha,beta);
    */
  }

  delta = new nonspatial_parm(DELTA,delta_init,delta_mean,delta_sd,delta_propsd,&uniform_dens,&log_transform);
  nu = new nonspatial_parm(NU,nu_init,nu_low,nu_high,nu_propsd,&uniform_dens,&identity);
  // initialize dataset - note that likelihood must know what the parameters are 
  if(prior==STATIONARY){
    data=new dataset(dat_fn_in,years_fn_in,YEAR,num_gridpoints,alpha->psi->value,beta->psi->value,eta->psi->value,&delta->psi->value,&nu->psi->value,&data_kappa->psi->value,0,0,0,0,0,"0");
    alpha->add_info();
    beta->add_info();
    eta->add_info();
  }
  if(prior==NON_STATIONARY){
    data=new dataset(dat_fn_in,years_fn_in,YEAR,num_gridpoints,alpha->psi->value,beta->psi->value,eta->psi->value,&delta->psi->value,&nu->psi->value,&data_kappa->psi->value,0,eigenval1vec->ptrvec,eigenval2vec->ptrvec,eigenvec1vec->ptrvec,0,"0");

    /* full kernel approach
    data=new dataset(dat_fn_in,years_fn_in,YEAR,num_gridpoints,alpha->psi->value,beta->psi->value,eta->psi->value,&delta->psi->value,0,0,eigenval1->psi->value,eigenval2->psi->value,eigenvec1->psi->value,eigenvec2->psi->value,"0");
    */
    alpha->add_info();
    beta->add_info();
    eta->add_info();
  }
  if(prior==FIXED){
    data=new dataset(dat_fn_in,years_fn_in,YEAR,num_gridpoints,alpha->psi->value,beta->psi->value,eta->psi->value,&delta->psi->value,&nu->psi->value,0,0,0,0,0,0,datacorr_fn_in);
    alpha->add_info();
    beta->add_info();
    eta->add_info();
  }

  storage0=new mat_pp<double>(storage_space,data->num_test_years+2+data->num_test_years+data->num_test_years);  // +2 is for sum and logpost determinant; 3 copies are for logpost, SSE, SSSE
}  

chain::~chain(void){
  delete storage0;
  delete storage1;
  delete storage2;
  delete storage3;
  delete storage4;
  if(prior==NON_STATIONARY){
    delete storage5;
    /* for outputting basis kernels
    delete storage6;
    */
    delete eigenval1vec;
    delete eigenval2vec;
    delete eigenvec1vec;
    delete data_kappa;
    // delete [] kernlocs;
    /* for full kernel approach
    delete storage6;
    delete storage7;
    delete eigenval1;
    delete eigenval2;
    delete eigenvec1;
    delete eigenvec2;
    */
  }
  if(prior==STATIONARY){
    delete data_kappa;
  }
  delete alpha; 
  delete beta;
  delete eta;
  delete delta;
  delete data;
}


void chain::do_a_step(int it_cur)
{
  int accepted=0;
  int mode=POSTMEAN; // SIMPLE; //POSTMEAN;
  int fastparm_fixed;
  // single round of parameter updating
  if(it_cur==50 || it_cur==9500 || it_cur==2250 || it_cur==2900){
    dumb();
  } 
  if(it_cur%fastparm_interval==0){
    fastparm_fixed=FALSE;
  }
  else{
    fastparm_fixed=TRUE;
  }
  alpha->step(mode,NON_LANGEVIN,JOINT_THETA,fastparm_fixed);
  beta->step(mode,NON_LANGEVIN,JOINT_THETA,fastparm_fixed);

  eta->step(mode,NON_LANGEVIN,JOINT_THETA,fastparm_fixed); // don't use Langevin for eta
  if(prior==NON_STATIONARY){
    // basis kernel approach
    // using POSTMEAN proposal seems unnecessary since basis kernel parameters mix fast
    // if I change to POSTMEAN, then move ksi_propose below should only depend on delta
    accepted+=data_kappa->step();
    // don't change basis kernel proposals to POSTMEAN unless I make sure that chi for alpha,beta,eta is recalculated
    accepted+=eigenval1vec->step(SIMPLE);
    accepted+=eigenval2vec->step(SIMPLE);
    accepted+=eigenvec1vec->step(SIMPLE);
    /* full kernel 
    eigenval1->step(SIMPLE,NON_LANGEVIN,JOINT_THETA);
    eigenval2->step(SIMPLE,NON_LANGEVIN,JOINT_THETA);
    eigenvec1->step(SIMPLE,NON_LANGEVIN,JOINT_THETA);
    eigenvec2->step(SIMPLE,NON_LANGEVIN,JOINT_THETA);
    */
  }
  if(!fastparm_fixed){
    accepted+=delta->step();
  }
  if(!(prior==FIXED)){
    accepted+=nu->step();
  }
  if(prior==STATIONARY){
    if(!fastparm_fixed){
      accepted+=data_kappa->step();
    }
  }
  // I do the chi recalc always because it's almost certain that eta or one of its parameters will change or that one of the basis kernels will change
  if(mode==POSTMEAN){  // this deals with updating chi since data cov has changed but for delta and data_kappa, the proposal was not joint with alpha and beta
    alpha->ksi_propose(TRUE);
    beta->ksi_propose(TRUE);
    eta->ksi_propose(TRUE);
  }
}


void chain::store(int it_cur)
{
  // stores parm values from current iteration
  int year;
  double logprior=0.0;
  double tmp, tmp2;
  int num=100;
  double SSalpha=0.0,SSbeta=0.0,SSeta=0.0;

  logprior=alpha->kappa->logprior+alpha->mu->logprior+alpha->sigma->logprior+alpha->theta->logprior+beta->kappa->logprior+beta->mu->logprior+beta->sigma->logprior+beta->theta->logprior+eta->kappa->logprior+eta->mu->logprior+eta->sigma->logprior+eta->theta->logprior+delta->psi->logprior+nu->psi->logprior;
  if(prior==STATIONARY || prior==NON_STATIONARY){
    logprior+=data_kappa->psi->logprior;
  }
  if(prior==NON_STATIONARY){
    for(int j=0;j<locations->num_kernels;j++){
      logprior+=eigenval1vec->psi[j]->logprior;
      logprior+=eigenval2vec->psi[j]->logprior;
      logprior+=eigenvec1vec->psi[j]->logprior;
    }
  }

  if(it_cur%num==0){  // change to (it_cur+1%num)
      cout << "Finished " << it_cur << " iterations.\n";

      // calculate and output acceptance rates

      // only used for LANGEVIN stuff
      if(alpha->oldgrad_exceeded+alpha->newgrad_exceeded+beta->oldgrad_exceeded+beta->newgrad_exceeded){
	cout << "Gradient truncated: " << alpha->oldgrad_exceeded << " " << alpha->newgrad_exceeded << " " << beta->oldgrad_exceeded << " " << beta->newgrad_exceeded << endl;
	alpha->oldgrad_exceeded=0;
	alpha->newgrad_exceeded=0;
	beta->oldgrad_exceeded=0;
	beta->newgrad_exceeded=0;
      }

     //      cout << setprecision(5) << "Current acceptance rates: " << data->loglik << " " << logpost << "\n";
      cout << setprecision(5) << "Current acceptance rates: " << data->loglik << " " << logprior << "\n";
    tmp=(double)alpha->num_accept_theta/it_cur;
    tmp2=(double) alpha->run_num_accept_theta/num;
    alpha->run_num_accept_theta=0;
    cout << setprecision(2) <<"A,T: " << tmp << "\t" << tmp2 << endl;
    tmp=(double)beta->num_accept_theta/it_cur;
    tmp2=(double) beta->run_num_accept_theta/num;
    beta->run_num_accept_theta=0;
    cout << setprecision(2) << "B,T: " << tmp << "\t" << tmp2 << endl;
    tmp=(double)eta->num_accept_theta/it_cur;
    tmp2=(double) eta->run_num_accept_theta/num;
    eta->run_num_accept_theta=0;
    cout << setprecision(2) << "E,T: " << tmp << "\t" << tmp2 << endl;

    if(alpha->num_accept_theta_subgroup){
      tmp=(double)alpha->num_accept_theta_subgroup/it_cur;
      tmp2=(double) alpha->run_num_accept_theta_subgroup/num;
      alpha->run_num_accept_theta_subgroup=0;
      cout << setprecision(2) <<"A,T: " << tmp << "\t" << tmp2 << endl;
    }
    if(beta->num_accept_theta_subgroup){
      tmp=(double)beta->num_accept_theta_subgroup/it_cur;
      tmp2=(double) beta->run_num_accept_theta_subgroup/num;
      beta->run_num_accept_theta_subgroup=0;
      cout << setprecision(2) << "B,T: " << tmp << "\t" << tmp2 << endl;
    }
    if(eta->num_accept_theta_subgroup){
      tmp=(double)eta->num_accept_theta_subgroup/it_cur;
      tmp2=(double) eta->run_num_accept_theta_subgroup/num;
      eta->run_num_accept_theta_subgroup=0;
      cout << setprecision(2) << "E,T: " << tmp << "\t" << tmp2 << endl;
    }

    tmp=(double)alpha->num_accept_mu/it_cur;
    tmp2=(double) alpha->run_num_accept_mu/num;
    alpha->run_num_accept_mu=0;
    cout << setprecision(2) << "A,M: " << tmp << "\t" << tmp2 << "\t" << alpha->mu->value << endl;
    tmp=(double)beta->num_accept_mu/it_cur;
    tmp2=(double) beta->run_num_accept_mu/num;
    beta->run_num_accept_mu=0;
    cout << setprecision(2) <<"B,M: " << tmp << "\t" << tmp2 << "\t" << beta->mu->value<< endl;
    tmp=(double)eta->num_accept_mu/it_cur;
    tmp2=(double) eta->run_num_accept_mu/num;
    eta->run_num_accept_mu=0;
    cout << setprecision(2) <<"E,M: " << tmp << "\t" << tmp2 << "\t" << eta->mu->value<< endl;
    tmp=(double)alpha->num_accept_sigma/it_cur;
    tmp2=(double) alpha->run_num_accept_sigma/num;
    alpha->run_num_accept_sigma=0;
     cout << setprecision(2) << "A,S: " << tmp << "\t" << tmp2 << "\t" << alpha->sigma->value<< endl;
    tmp=(double)beta->num_accept_sigma/it_cur;
    tmp2=(double) beta->run_num_accept_sigma/num;
    beta->run_num_accept_sigma=0;
    cout << setprecision(2) <<"B,S: " << tmp << "\t" << tmp2 << "\t" << beta->sigma->value<< endl;
    tmp=(double)eta->num_accept_sigma/it_cur;
    tmp2=(double) eta->run_num_accept_sigma/num;
    eta->run_num_accept_sigma=0;
    cout << setprecision(2) <<"E,S: " << tmp << "\t" << tmp2 << "\t" << eta->sigma->value<< endl;
     tmp=(double) alpha->num_accept_kappa/it_cur;
    tmp2=(double) alpha->run_num_accept_kappa/num;
    alpha->run_num_accept_kappa=0;
    cout << setprecision(2) << "A,K: " << tmp << "\t" << tmp2 << "\t" << alpha->kappa->value<< endl;
    tmp=(double)beta->num_accept_kappa/it_cur;
    tmp2=(double) beta->run_num_accept_kappa/num;
    beta->run_num_accept_kappa=0;
    cout << setprecision(2) <<"B,K: " << tmp << "\t" << tmp2 << "\t" << beta->kappa->value<< endl;
    tmp=(double)eta->num_accept_kappa/it_cur;
    tmp2=(double) eta->run_num_accept_kappa/num;
    eta->run_num_accept_kappa=0;
    cout << setprecision(2) <<"E,K: " << tmp << "\t" << tmp2 << "\t" << eta->kappa->value << endl;

    // kappa on its own
     tmp=(double) alpha->num_accept_kappa2/it_cur;
    tmp2=(double) alpha->run_num_accept_kappa2/num;
    alpha->run_num_accept_kappa2=0;
    cout << setprecision(2) << "A,K2: " << tmp << "\t" << tmp2 << "\t" << alpha->kappa->value<< endl;
    tmp=(double)beta->num_accept_kappa2/it_cur;
    tmp2=(double) beta->run_num_accept_kappa2/num;
    beta->run_num_accept_kappa2=0;
    cout << setprecision(2) <<"B,K2: " << tmp << "\t" << tmp2 << "\t" << beta->kappa->value<< endl;
    tmp=(double)eta->num_accept_kappa2/it_cur;
    tmp2=(double) eta->run_num_accept_kappa2/num;
    eta->run_num_accept_kappa2=0;
    cout << setprecision(2) <<"E,K2: " << tmp << "\t" << tmp2 << "\t" << eta->kappa->value << endl;

   tmp=(double)delta->num_accept/it_cur;
    tmp2=(double) delta->run_num_accept/num;
    delta->run_num_accept=0;
    cout << setprecision(2) << "D  : " << tmp << "\t" << tmp2 << "\t" << delta->psi->value << endl;
   tmp=(double)nu->num_accept/it_cur;
    tmp2=(double) nu->run_num_accept/num;
    nu->run_num_accept=0;
    cout << setprecision(2) << "V  : " << tmp << "\t" << tmp2 << "\t" << nu->psi->value << endl;
    if(prior==STATIONARY){
      tmp=(double)data_kappa->num_accept/it_cur;
      tmp2=(double) data_kappa->run_num_accept/num;
      data_kappa->run_num_accept=0;
      cout << setprecision(2) << "K  : " << tmp << "\t" << tmp2 << "\t" << data_kappa->psi->value<< endl;
    }
    if(prior==NON_STATIONARY){

      tmp=(double)data_kappa->num_accept/it_cur;
      tmp2=(double) data_kappa->run_num_accept/num;
      data_kappa->run_num_accept=0;
      cout << setprecision(2) << "K  : " << tmp << "\t" << tmp2 << "\t" << data_kappa->psi->value <<  endl;

      tmp=(double)eigenval1vec->num_accept/it_cur;
      tmp2=(double) eigenval1vec->run_num_accept/num;
      eigenval1vec->run_num_accept=0;
      cout << setprecision(2) << "L1:  " << tmp << "\t" << tmp2 << "\t" << eigenval1vec->psi[0]->value << endl;

      tmp=(double)eigenval2vec->num_accept/it_cur;
      tmp2=(double) eigenval2vec->run_num_accept/num;
      eigenval2vec->run_num_accept=0;
      cout << setprecision(2) << "L2:  " << tmp << "\t" << tmp2 << "\t" << eigenval2vec->psi[0]->value << endl;

      tmp=(double)eigenvec1vec->num_accept/it_cur;
      tmp2=(double) eigenvec1vec->run_num_accept/num;
      eigenvec1vec->run_num_accept=0;
      cout << setprecision(2) << "G1:  " << tmp << "\t" << tmp2 << "\t" << eigenvec1vec->psi[0]->value << endl;

      /* full kernel approach
      tmp=(double)eigenval1->num_accept_theta/it_cur;
      tmp2=(double) eigenval1->run_num_accept_theta/num;
      eigenval1->run_num_accept_theta=0;
      cout << setprecision(2) << "L1,T: " << tmp << "\t" << tmp2 << endl;

      if(eigenval1->num_accept_theta_subgroup){
	tmp=(double)eigenval1->num_accept_theta_subgroup/it_cur;
	tmp2=(double) eigenval1->run_num_accept_theta_subgroup/num;
	eigenval1->run_num_accept_theta_subgroup=0;
	cout << setprecision(2) << "L1,T: " << tmp << "\t" << tmp2 << endl;
      }
      tmp=(double)eigenval1->num_accept_mu/it_cur;
      tmp2=(double) eigenval1->run_num_accept_mu/num;
      eigenval1->run_num_accept_mu=0;
      cout << setprecision(2) << "L1,M: " << tmp << "\t" << tmp2 << endl;
      tmp=(double)eigenval1->num_accept_sigma/it_cur;
      tmp2=(double) eigenval1->run_num_accept_sigma/num;
      eigenval1->run_num_accept_sigma=0;
      cout << setprecision(2) << "L1,S: " << tmp << "\t" << tmp2 << endl;
      tmp=(double) eigenval1->num_accept_kappa/it_cur;
      tmp2=(double) eigenval1->run_num_accept_kappa/num;
      eigenval1->run_num_accept_kappa=0;
      cout << setprecision(2) << "L1,K: " << tmp << "\t" << tmp2 << endl;
      
      tmp=(double)eigenval2->num_accept_theta/it_cur;
      tmp2=(double) eigenval2->run_num_accept_theta/num;
      eigenval2->run_num_accept_theta=0;
      cout << setprecision(2) << "L2,T: " << tmp << "\t" << tmp2 << endl;
      if(eigenval2->num_accept_theta_subgroup){
	tmp=(double)eigenval2->num_accept_theta_subgroup/it_cur;
	tmp2=(double) eigenval2->run_num_accept_theta_subgroup/num;
	eigenval2->run_num_accept_theta_subgroup=0;
	cout << setprecision(2) << "L2,T: " << tmp << "\t" << tmp2 << endl;
      }
      tmp=(double)eigenval2->num_accept_mu/it_cur;
      tmp2=(double) eigenval2->run_num_accept_mu/num;
      eigenval2->run_num_accept_mu=0;
      cout << setprecision(2) << "L2,M: " << tmp << "\t" << tmp2 << endl;
      tmp=(double)eigenval2->num_accept_sigma/it_cur;
      tmp2=(double) eigenval2->run_num_accept_sigma/num;
      eigenval2->run_num_accept_sigma=0;
      cout << setprecision(2) << "L2,S: " << tmp << "\t" << tmp2 << endl;
      tmp=(double) eigenval2->num_accept_kappa/it_cur;
      tmp2=(double) eigenval2->run_num_accept_kappa/num;
      eigenval2->run_num_accept_kappa=0;
      cout << setprecision(2) << "L2,K: " << tmp << "\t" << tmp2 << endl;

      tmp=(double)eigenvec1->num_accept_theta/it_cur;
      tmp2=(double) eigenvec1->run_num_accept_theta/num;
      eigenvec1->run_num_accept_theta=0;
      cout << setprecision(2) << "G1,T: " << tmp << "\t" << tmp2 << endl;
      if(eigenvec1->num_accept_theta_subgroup){
	tmp=(double)eigenvec1->num_accept_theta_subgroup/it_cur;
	tmp2=(double) eigenvec1->run_num_accept_theta_subgroup/num;
	eigenvec1->run_num_accept_theta_subgroup=0;
	cout << setprecision(2) << "G1,T: " << tmp << "\t" << tmp2 << endl;
      }
      tmp=(double)eigenvec1->num_accept_mu/it_cur;
      tmp2=(double) eigenvec1->run_num_accept_mu/num;
      eigenvec1->run_num_accept_mu=0;
      cout << setprecision(2) << "G1,M: " << tmp << "\t" << tmp2 << endl;
      tmp=(double)eigenvec1->num_accept_sigma/it_cur;
      tmp2=(double) eigenvec1->run_num_accept_sigma/num;
      eigenvec1->run_num_accept_sigma=0;
      cout << setprecision(2) << "G1,S: " << tmp << "\t" << tmp2 << endl;
      tmp=(double) eigenvec1->num_accept_kappa/it_cur;
      tmp2=(double) eigenvec1->run_num_accept_kappa/num;
      eigenvec1->run_num_accept_kappa=0;
      cout << setprecision(2) << "G1,K: " << tmp << "\t" << tmp2 << endl;
      
      tmp=(double)eigenvec2->num_accept_theta/it_cur;
      tmp2=(double) eigenvec2->run_num_accept_theta/num;
      eigenvec2->run_num_accept_theta=0;
      cout << setprecision(2) << "G2,T: " << tmp << "\t" << tmp2 << endl;
      if(eigenvec2->num_accept_theta_subgroup){
	tmp=(double)eigenvec2->num_accept_theta_subgroup/it_cur;
	tmp2=(double) eigenvec2->run_num_accept_theta_subgroup/num;
	eigenvec2->run_num_accept_theta_subgroup=0;
	cout << setprecision(2) << "G2,T: " << tmp << "\t" << tmp2 << endl;
      }
      tmp=(double)eigenvec2->num_accept_mu/it_cur;
      tmp2=(double) eigenvec2->run_num_accept_mu/num;
      eigenvec2->run_num_accept_mu=0;
      cout << setprecision(2) << "G2,M: " << tmp << "\t" << tmp2 << endl;
      tmp=(double)eigenvec2->num_accept_sigma/it_cur;
      tmp2=(double) eigenvec2->run_num_accept_sigma/num;
      eigenvec2->run_num_accept_sigma=0;
      cout << setprecision(2) << "G2,S: " << tmp << "\t" << tmp2 << endl;
      tmp=(double) eigenvec2->num_accept_kappa/it_cur;
      tmp2=(double) eigenvec2->run_num_accept_kappa/num;
      eigenvec2->run_num_accept_kappa=0;
      cout << setprecision(2) << "G2,K: " << tmp << "\t" << tmp2 << endl;
      */
    }
  }

  if(it_cur%storage_interval==0){
    // store log posterior at test time points at same interval as parm values
    logpost=data->evaluate_logpost();
    storage0->vals[it_run][0]=logpost;
    storage0->vals[it_run][1]=data->logpost_det;
    for(year=0;year<data->num_test_years;year++){
      storage0->vals[it_run][year+2]=data->logpost_test[year];
    }
    for(year=0;year<data->num_test_years;year++){
      storage0->vals[it_run][data->num_test_years+year+2]=data->SSE_test[year];
    }
    for(year=0;year<data->num_test_years;year++){
      storage0->vals[it_run][data->num_test_years+data->num_test_years+year+2]=data->SSSE_test[year];
    }
    storage1->vals[it_run][0]=data->loglik;
    storage1->vals[it_run][1]=logprior;

    storage1->vals[it_run][2]=alpha->mu->value;
    storage1->vals[it_run][3]=alpha->sigma->value;
    storage1->vals[it_run][4]=alpha->kappa->value;
    storage1->vals[it_run][5]=beta->mu->value;
    storage1->vals[it_run][6]=beta->sigma->value;
    storage1->vals[it_run][7]=beta->kappa->value;
    storage1->vals[it_run][8]=eta->mu->value;
    storage1->vals[it_run][9]=eta->sigma->value;
    storage1->vals[it_run][10]=eta->kappa->value;
    storage1->vals[it_run][11]=delta->psi->value;
    storage1->vals[it_run][12]=nu->psi->value;

    for(int j=0;j<num_gridpoints;j++){
      SSalpha+=pow(alpha->theta->value[j],2);
      SSbeta+=pow(beta->theta->value[j],2);
      SSeta+=pow(eta->theta->value[j],2);
    }
    // data covariance storage
    if(prior==STATIONARY){
      storage1->vals[it_run][13]=data_kappa->psi->value;
      storage1->vals[it_run][14]=SSalpha;
      storage1->vals[it_run][15]=SSbeta;
      storage1->vals[it_run][16]=SSeta;
    }
    if(prior==FIXED){
      storage1->vals[it_run][13]=0.0;
      storage1->vals[it_run][14]=SSalpha;
      storage1->vals[it_run][15]=SSbeta;
      storage1->vals[it_run][16]=SSeta;
    }
    if(prior==NON_STATIONARY){
      storage1->vals[it_run][13]=0.0;
      storage1->vals[it_run][14]=SSalpha;
      storage1->vals[it_run][15]=SSbeta;
      storage1->vals[it_run][16]=SSeta;

     // data covariance storage

      storage5->vals[it_run][0]=data_kappa->psi->value;
      storage5->vals[it_run][1]=data_kappa->psi->value;
      int cnt=2;
      for(int j=0;j<locations->num_kernels;j++){
	storage5->vals[it_run][cnt]=eigenval1vec->psi[j]->value;
        cnt++;
      }
      for(int j=0;j<locations->num_kernels;j++){
	storage5->vals[it_run][cnt]=eigenval2vec->psi[j]->value;
        cnt++;
      }
      for(int j=0;j<locations->num_kernels;j++){
	storage5->vals[it_run][cnt]=eigenvec1vec->psi[j]->value;
        cnt++;
      }

      /*
      int permlocn;
      cnt=0;
      for(int j=0;j<15;j++){
        permlocn=locations->invp[kernlocs[j]-1]-PERMSTART1;
        storage6->vals[it_run][cnt]=data->nonstat_cov->kernels->vals[permlocn][0]; cnt++;
        storage6->vals[it_run][cnt]=data->nonstat_cov->kernels->vals[permlocn][1]; cnt++;
        storage6->vals[it_run][cnt]=data->nonstat_cov->kernels->vals[permlocn][2]; cnt++;
        storage6->vals[it_run][cnt]=data->nonstat_cov->kernels->vals[permlocn][3]; cnt++;
      }
      */

      /* full kernel approach
      storage1->vals[it_run][12]=0.0;
      for(int j=0;j<num_gridpoints;j++){
	storage5->vals[it_run][j]=eigenval1->psi->value[j];
	storage5->vals[it_run][j+num_gridpoints]=eigenval2->psi->value[j];
      }
      for(int j=0;j<num_gridpoints;j++){
	storage6->vals[it_run][j]=eigenvec1->psi->value[j];
	storage6->vals[it_run][j+num_gridpoints]=eigenvec2->psi->value[j];
      }
      storage7->vals[it_run][0]=eigenval1->mu->value;
      storage7->vals[it_run][1]=eigenval1->sigma->value;
      storage7->vals[it_run][2]=eigenval1->kappa->value;
      storage7->vals[it_run][3]=eigenval2->mu->value;
      storage7->vals[it_run][4]=eigenval2->sigma->value;
      storage7->vals[it_run][5]=eigenval2->kappa->value;
      storage7->vals[it_run][6]=eigenvec1->mu->value;
      storage7->vals[it_run][7]=eigenvec1->sigma->value;
      storage7->vals[it_run][8]=eigenvec1->kappa->value;
      storage7->vals[it_run][9]=eigenvec2->mu->value;
      storage7->vals[it_run][10]=eigenvec2->sigma->value;
      storage7->vals[it_run][11]=eigenvec2->kappa->value;
      */
    }
    
    for(int j=0;j<num_gridpoints;j++){
      storage2->vals[it_run][j]=alpha->psi->value[j];
    }
    for(int j=0;j<num_gridpoints;j++){
      storage3->vals[it_run][j]=beta->psi->value[j];
    }
    for(int j=0;j<num_gridpoints;j++){
      storage4->vals[it_run][j]=eta->psi->value[j];
    }
    
    it_run++;
  }
  

  if(it_run>0 && it_run%storage_space==0){
    report(it_cur,CURRENT);
  }
}
  
void chain::report(int it_cur,int status){
  cout << "Process: wait. Writing output or final values at iteration " << it_cur << " to files " << out_fn << "; ";
  time_t t1=time(0);
  char* timeout=ctime(&t1);
  cout << timeout << endl; cout.flush();
  if(it_run>0){
    it_run=0;

    string out_fn_tmp;
    ofstream output;
    
    //string blocks=itos(block);

    string blocks="";  // I'm appending, so don't need 'blocks' to indicate which file
    out_fn_tmp=out_fn+"0" + blocks;
    output.open(out_fn_tmp.c_str(),ios::app);
    output << storage0;
    output.close();

    out_fn_tmp=out_fn+"1" + blocks;  // formerly "1."
    output.open(out_fn_tmp.c_str(),ios::app);  // formerly ios::out
    output << storage1;
    output.close();

    out_fn_tmp=out_fn+"2" + blocks;
    output.open(out_fn_tmp.c_str(),ios::app);
    output << storage2;
    output.close();

    out_fn_tmp=out_fn+"3" + blocks;
    output.open(out_fn_tmp.c_str(),ios::app);
    output << storage3;
    output.close();

    out_fn_tmp=out_fn+"4" + blocks;
    output.open(out_fn_tmp.c_str(),ios::app);
    output << storage4;
    output.close();

    if(prior==NON_STATIONARY){
      out_fn_tmp=out_fn+"5" + blocks;
      output.open(out_fn_tmp.c_str(),ios::app);
      output << storage5;
      output.close();

      // for subset of kernels
      /*
      out_fn_tmp=out_fn+"6" + blocks;
      output.open(out_fn_tmp.c_str(),ios::app);
      output << storage6;
      output.close();
      */

      /* only for full kernel approach
      out_fn_tmp=out_fn+"6." + blocks;
      output.open(out_fn_tmp.c_str(),ios::out);
      output << storage6;
      output.close();
      
      out_fn_tmp=out_fn+"7." + blocks;
      output.open(out_fn_tmp.c_str(),ios::out);
      output << storage7;
      output.close();
      */
    }
    block++;
  }

    // storage and output will both contain permuted versions -- unpermute when analyzing as necessary
  
    // final report and transfer of stored iterates to a file

  //if(status==FINAL){

      ofstream final(final_fn.c_str());
      final << setprecision(6);
      
      final << 2 << " ";       // indiv theta init, i.e., no repeat initialization
      for(int j=0;j<num_gridpoints;j++){
	final << alpha->theta->value[j] << " ";
      }
      final << endl << alpha->mu->base_value << endl; 
      final << alpha->sigma->base_value << endl;
      final << alpha->kappa->base_value << endl;
      
      final << 2 << " ";       // indiv theta init, i.e., no repeat initialization
      for(int j=0;j<num_gridpoints;j++){
	final << beta->theta->value[j] << " ";
      }
      final << endl << beta->mu->base_value << endl; 
      final << beta->sigma->base_value << endl;
      final << beta->kappa->base_value << endl;
      
      final << 2 << " ";       // indiv theta init, i.e., no repeat initialization
      for(int j=0;j<num_gridpoints;j++){
	final << eta->theta->value[j] << " ";
      }
      final << endl << eta->mu->base_value << endl; 
      final << eta->sigma->base_value << endl;
      final << eta->kappa->base_value << endl;
      
      final << delta->psi->base_value << endl;
      final << nu->psi->base_value << endl;

      if(prior==STATIONARY){
	final << data_kappa->psi->base_value << endl;
      }
      if(prior==NON_STATIONARY){

	final << data_kappa->psi->base_value << endl;

	final << 2 << " ";       // no repeat initialization
	for(int j=0;j<locations->num_kernels;j++){
	  final << eigenval1vec->psi[j]->base_value << " ";
	}
	final << endl;
	final << 2 << " ";       // no repeat initialization
	for(int j=0;j<locations->num_kernels;j++){
	  final << eigenval2vec->psi[j]->base_value << " ";
	}
	final << endl;
	final << 2 << " ";       // no repeat initialization
	for(int j=0;j<locations->num_kernels;j++){
	  final << eigenvec1vec->psi[j]->base_value << " ";
	}
	final << endl;

        /* full kernel approach
	final << 2 << " ";       // no repeat initialization
	for(int j=0;j<num_gridpoints;j++){
	  final << eigenval1->theta->value[j] << " ";
	}
	final << endl << eigenval1->mu->base_value << endl; 
	final << eigenval1->sigma->base_value << endl;
	final << eigenval1->kappa->base_value << endl;

	final << 2 << " ";       // no repeat initialization
	for(int j=0;j<num_gridpoints;j++){
	  final << eigenval2->theta->value[j] << " ";
	}
	final << endl << eigenval2->mu->base_value << endl; 
	final << eigenval2->sigma->base_value << endl;
	final << eigenval2->kappa->base_value << endl;

	final << 2 << " ";       // no repeat initialization
	for(int j=0;j<num_gridpoints;j++){
	  final << eigenvec1->theta->value[j] << " ";
	}
	final << endl <<eigenvec1->mu->base_value << endl; 
	final << eigenvec1->sigma->base_value << endl;
	final <<  eigenvec1->kappa->base_value << endl;

	final << 2 << " ";       // no repeat initialization
	for(int j=0;j<num_gridpoints;j++){
	  final << eigenvec2->theta->value[j] << " ";
	}
	final << endl << eigenvec2->mu->base_value << endl; 
	final << eigenvec2->sigma->base_value << endl;
	final << eigenvec2->kappa->base_value << endl;
        */
      }
      char line[1024];
      for(int j=0;j<1024;j++){
        line[j]=' ';
      }
      ran1(idum,WRITE,line);
      final << line << endl;
      final.close();

      cout << "Process: done writing.\n"; cout.flush();
}







